@@ -1,4 +1,3 @@
-/*Paraselene generated this source file.*/
 function paraselene_get_id(id){
 	if(document.getElementById){
 		return document.getElementById(id);
@@ -139,7 +138,7 @@ function paraselene_get_event(ev){
 	return window.event;
 }
 
-function paraselene_tag_set_event(dialog,tag){
+function paraselene_dialog_set_event(dialog,tag){
 	dialog.onmousedown=function(kmouse){
 		paraselene_z_order(dialog,false);
 		return true;
@@ -361,7 +360,7 @@ function paraselene_back_kill(){
 
 function paraselene_back_kill_start(frg){
 	if(paraselene_back_kill_f)return;
-	document.location.href=document.location.href+"#"+frg;
+	document.location.href=document.location.href.split('#')[0]+"#"+frg;
 	paraselene_back_kill_f=true;
 	paraselene_back_kill();
 }
@@ -488,10 +487,10 @@ function paraselene_dialog_open(dialog){
 	base.style.borderColor=dialog.border;
 
 	var head=document.createElement("div");
-	head.className="paraselenePopupHead";
+	head.className="paraselenePopupHead paraseleneGradient";
 	head.style.borderColor=dialog.border;
 
-	var a=document.createElement("a");
+	var a=document.createElement("span");
 	a.className="paraselenePopupHead";
 	a.innerHTML=dialog.title;
 	head.appendChild(a);
@@ -516,7 +515,7 @@ function paraselene_dialog_open(dialog){
 	point.appendChild(base);
 	paraselene_tag_center(base,paraselene_mous_up_x,paraselene_mous_up_y);
 	paraselene_back_kill_start(pageid);
-	paraselene_tag_set_event(base,head);
+	paraselene_dialog_set_event(base,head);
 }
 
 function paraselene_action(obj){
@@ -576,10 +575,9 @@ function paraselene_exec(exec){
 
 function paraselene_url_check(url,param){
 	if(!navigator.cookieEnabled)	return undefined;
-	var jsession=url.split(";jsessionid");
-	url=jsession[0];
 	var url_test=url.split("?");
-	url=url_test[0];
+	var jsession=url_test[0].split(";jsessionid");
+	url=jsession[0];
 
 	if(url_test[1]){
 		url=url+"?"+url_test[1];
