@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -13,6 +13,7 @@ import java.net.*;
 import javax.servlet.http.*;
 import paraselene.tag.*;
 import paraselene.css.*;
+import paraselene.ui.*;
 import paraselene.supervisor.*;
 import paraselene.tag.form.*;
 import paraselene.ajax.data.*;
@@ -181,6 +182,10 @@ public abstract class Page implements Serializable {
 	private HashMap<String,HashMap<String,Tag>>	tag_m_dic = new HashMap<String,HashMap<String,Tag>>();
 	private HashMap<Tag,String>	name_dic = new HashMap<Tag,String>();
 
+	private ArrayList<PageBypassedDiv>	bypass = new ArrayList<PageBypassedDiv>();
+	private ArrayList<PageHooker>	hook = new ArrayList<PageHooker>();
+	private ArrayList<Clickable>	clickable = new ArrayList<Clickable>();
+
 	private int name_seq = 0;
 	private Tag	contents = null;
 	private boolean init_f = true;
@@ -195,6 +200,22 @@ public abstract class Page implements Serializable {
 	private static final Object	obj4uniKey = new Object();
 	private String uniKey;
 
+	/**
+	 * 保持している PageBypassedDiv を返します。
+	 * @return PageBypassedDivの配列。無ければ 0 個の配列を返します。
+	 */
+	public PageBypassedDiv[] getPageBypassedDiv() {
+		return bypass.toArray( new PageBypassedDiv[0] );
+	}
+
+	/**
+	 * 保持している PageHooker を返します。
+	 * @return PageHookerの配列。無ければ 0 個の配列を返します。
+	 */
+	public PageHooker[] getPageHooker() {
+		return hook.toArray( new PageHooker[0] );
+	}
+
 	private void makeKey() {
 		long	now = 0;
 		synchronized( obj4uniKey ) {
@@ -377,32 +398,76 @@ public abstract class Page implements Serializable {
 		return modified;
 	}
 
+	private Clickable	clicked_tag = null;
+
+	/**
+	 * 押下タグの取得。
+	 * @return 押下タグ。無ければnull。
+	 */
+	public Clickable getClickedTag() {
+		if ( clicked_tag == null )	return null;
+		if ( clicked_tag.isClicked() )	return clicked_tag;
+		return null;
+	}
+
+	/**
+	 * 押下ボタンの取得。
+	 * @return 押下ボタン。無ければnull。
+	 */
+	public Button getClickedButton() {
+		Clickable	c = getClickedTag();
+		if ( c instanceof Button ) {
+			return (Button)c;
+		}
+		return null;
+	}
+
 	/**
 	 * リクエスト内容をフォームに反映する。
 	 * input()メソッドを呼ぶ直前にはreflect()がコールされています。
 	 * 通常のコーディングであればプログラマはこのメソッドを使う事はありません。
 	 * @param req リクエスト。
+	 * @return リクエスト内容を反映したページ。
 	 */
-	public void reflect( RequestParameter req ) {
+	public Page reflect( RequestParameter req ) {
+		clicked_tag = null;
+		int	click_cnt = clickable.size();
+		for ( int i = 0; i < click_cnt; i++ ) {
+			Clickable	c = clickable.get( i );
+			c.setClicked( false );
+		}
+
+		PageBypassedDiv[]	div = getPageBypassedDiv();
+		for ( int i = 0; i < div.length; i++ ) {
+			Page	page = div[i].getBypassPage();
+			if ( page != null ) {
+				page = page.reflect( req );
+				if ( page != null )	return page;
+			}
+		}
 		modified = new Control[0];
-		if ( getMainTag() == null )	return;
+		if ( getMainTag() == null )	return null;
+
+		for ( int i = 0; i < click_cnt; i++ ) {
+			Clickable	c = clickable.get( i );
+			if ( c.isClicked( req ) ) {
+				clicked_tag = c;
+				c.setClicked( true );
+				break;
+			}
+		}
+		if ( clicked_tag instanceof Anchor )	return this;
+
 		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
 		int	form_cnt = form.size();
 		String	form_id = "paraselene_form_id";	// 不一致用ダミー
 		RequestItem	item = req.getItem( Form.FORM_ID );
 		if ( item != null ) {
-			form_id = item.getValue( 0 );
+			form_id = item.getValue();
 		}
 		for ( int i = form_cnt - 1; i >= 0; i-- ) {
 			Form	f = form.get( i );
 			if ( !f.getID().equals( form_id ) ) {
-				Control[]	ctl = f.getAllControl();
-				for ( int j = 0; j < ctl.length; j++ ) {
-					if ( ctl[j] instanceof Button ) {
-						Button	but = (Button)ctl[j];
-						but.resetClicked();
-					}
-				}
 				form.remove( i );
 			}
 		}
@@ -420,24 +485,9 @@ public abstract class Page implements Serializable {
 				Option.debug( e );
 			}
 		}
-	}
 
-	/**
-	 * 押下ボタンの取得。
-	 * @return 押下ボタン。無ければnull。
-	 */
-	public Button getClickedButton() {
-		if ( getMainTag() == null )	return null;
-		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
-		int	size = form.size();
-		for ( int i = 0; i < size; i++ ) {
-			Control[]	ctl = form.get( i ).getAllControl();
-			for ( int j = 0; j < ctl.length; j++ ) {
-				if ( !(ctl[j] instanceof Button) )	continue;
-				Button	btn = (Button)ctl[j];
-				if ( btn.isClicked() ) return btn;
-			}
-		}
+		if ( getClickedTag() != null )	return this;
+		if ( form_cnt > 0 )	return this;
 		return null;
 	}
 
@@ -731,6 +781,16 @@ public abstract class Page implements Serializable {
 	public void nameTag( Tag tag ) {
 		if ( getMainTag() == null )	return;
 		tag.setAssignedPage( this );
+		if ( tag instanceof PageHooker ) {
+			hook.add( (PageHooker)tag );
+		}
+		if ( tag instanceof PageBypassedDiv ) {
+			bypass.add( (PageBypassedDiv)tag );
+		}
+		if ( tag instanceof Clickable ) {
+			clickable.add( (Clickable)tag );
+		}
+
 		String	name = tag.getNameAttribute();
 		if ( name != null ) {
 			if ( Form.FORM_ID.equals( name ) )	return;
@@ -777,6 +837,15 @@ public abstract class Page implements Serializable {
 			}
 		}
 		tag.setAssignedPage( null );
+		if ( tag instanceof PageHooker ) {
+			hook.remove( (PageHooker)tag );
+		}
+		if ( tag instanceof PageBypassedDiv ) {
+			bypass.remove( (PageBypassedDiv)tag );
+		}
+		if ( tag instanceof Clickable ) {
+			clickable.remove( (Clickable)tag );
+		}
 		String	name = tag.getNameAttribute();
 		if ( name == null )	return;
 		clearDic( name );
@@ -914,12 +983,12 @@ public abstract class Page implements Serializable {
 	 */
 	public void addOnLoadScript( String ... script ) {
 		paraselene.ajax.data.JavaScript	js = null;
-		if ( onload.getHTMLPartCount() == 0 ) {
+		if ( getOnloadTag().getHTMLPartCount() == 0 ) {
 			js = new paraselene.ajax.data.JavaScript( script );
-			onload.addHTMLPart( js );
+			getOnloadTag().addHTMLPart( js );
 			return;
 		}
-		js = (paraselene.ajax.data.JavaScript)onload.getHTMLPart( 0 );
+		js = (paraselene.ajax.data.JavaScript)getOnloadTag().getHTMLPart( 0 );
 		js.append( script );
 	}
 
@@ -928,7 +997,7 @@ public abstract class Page implements Serializable {
 	 * @return JavaScript。
 	 */
 	public String getOnLoadScript() {
-		return onload.getHTMLPart( 0 ).toString();
+		return getOnloadTag().getHTMLPart( 0 ).toString();
 	}
 
 	/**
@@ -937,8 +1006,8 @@ public abstract class Page implements Serializable {
 	 * 不用意にクリアしないで下さい。
 	 */
 	public void resetOnLoadScript() {
-		onload.removeHTMLPart();
-		onload.addHTMLPart( new paraselene.ajax.data.JavaScript() );
+		getOnloadTag().removeHTMLPart();
+		getOnloadTag().addHTMLPart( new paraselene.ajax.data.JavaScript() );
 	}
 
 	/**
@@ -1120,9 +1189,7 @@ public abstract class Page implements Serializable {
 					new Attribute( "id", AJAX_ADD_ID )
 				);
 				body.removeHTMLPart();
-				body.addHTMLPart( div[0], div[1], onload /*, Tag.makeScript(
-					"paraselene_add_event(paraselene_get_id('paraseleneDragonsTail'),'load',setTimeout(paraselene_form_comp,2000);"
-				) */ );
+				body.addHTMLPart( div[0], div[1], getOnloadTag() );
 			}
 			else	ajax_f = false;
 		}
@@ -1137,9 +1204,15 @@ public abstract class Page implements Serializable {
 	private RequestParameter	now_req;
 	private RequestParameter.Mobile			access_mobile;
 	private RequestParameter.SearchEngine	access_se;
-	private Tag	onload = Tag.makeScript();
+	private Tag	onload_tag = null;
 	private boolean safari_f = false;
 
+	private Tag getOnloadTag() {
+		if ( onload_tag != null )	return onload_tag;
+		onload_tag = Tag.makeScript();
+		return onload_tag;
+	}
+
 	/**
 	 * Ajax機能が利用可能であるか？以下の全てを満たす場合に true を返します。
 	 * <ul>
@@ -1217,5 +1290,28 @@ public abstract class Page implements Serializable {
 	 */
 	public boolean isNoCache() { return false; }
 
+	/**
+	 * titleタグ設定値の取得。
+	 * @return タイトル。無ければ空文字列を返します。
+	 */
+	public String getTitleString() {
+		Tag	tag = getFirstTagByType( "title" );
+		if ( tag == null )	return "";
+		String	title = tag.getValueString();
+		if ( title == null )	return "";
+		return title;
+	}
+
+	/**
+	 * ページタイトルラベル。
+	 * この戻り値がポップアップダイアログのタイトル部等に使用されます。<br>
+	 * これをオーバーライドする事で、その表示内容を変更する事ができます。
+	 * @return getTitleString()の戻り値から Text を生成して返します。
+	 */
+	public HTMLPart[] getTitleLabel() {
+		String	str = getTitleString();
+		if ( str.isEmpty() )	str = "　";
+		return new HTMLPart[]{ new Text( str ) };
+	}
 }
 
