@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -64,9 +64,13 @@ public class Open extends TagData {
 		bg = toBg( st );
 		bg2 = toBg( shift( c, 48 ) );
 		border = toBg( shift( c, 24 ) );
-		Tag	title_tag = page.getFirstTagByType( "title" );
-		if ( title_tag != null )	title = title_tag.getValueString();
-		if ( title == null )	title = "　";
+
+		HTMLPart[]	title_part = page.getTitleLabel();
+		StringBuilder	buf = new StringBuilder();
+		for ( int i = 0; i < title_part.length; i++ ) {
+			buf = buf.append( title_part[i].toString( HTMLPart.StringMode.BODY ) );
+		}
+		title = buf.toString();
 		modal = page.getPopupType() == Popup.Type.MODAL;
 		active = act_f;
 	}
