@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -17,7 +17,7 @@ import paraselene.*;
  * 遷移先指定。
  */
 public class Forward implements Serializable {
-	private static final long serialVersionUID = 2L;
+	private static final long serialVersionUID = 212L;
 	private int	error = 0;
 	private URI	redirect_uri = null;
 	PageID	out_page = null;
@@ -30,6 +30,7 @@ public class Forward implements Serializable {
 	ArrayList<Cookie>	out_cookie = new ArrayList<Cookie>();
 	boolean done_f = false;
 
+
 	boolean isLeave() {
 		return error > 0 || redirect_uri != null;
 	}
@@ -158,7 +159,8 @@ public class Forward implements Serializable {
 			if ( af[i] == null )	continue;
 			list.add( af[i] );
 		}
-		return list.toArray( new AjaxForward[0] );
+		af = list.toArray( new AjaxForward[0] );
+		return af;
 	}
 
 	private AjaxForward getNotAjax() {
