@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -139,6 +139,7 @@ public class RequestParameter implements Serializable {
 	private Method	method = Method.GET;
 	private URI	uri = null;
 	private URI	sub_uri = null;
+	private String	context_path = null;
 	private HttpSession	session = null;
 	private Cookie[]	in_cookie = null;
 	private String	remote_addr = null;
@@ -249,7 +250,7 @@ public class RequestParameter implements Serializable {
 	}
 
 	/**
-	 * リクエストURI設定。
+	 * リクエストURI設定。クエリも含めたフルパスが設定されます。
 	 * @param u URI。
 	 */
 	protected void setURI( URI u ) {
@@ -264,6 +265,23 @@ public class RequestParameter implements Serializable {
 		sub_uri = u;
 	}
 
+
+	/**
+	 * {@link HttpServletRequest#getContextPath()} を返します。
+	 * @return コンテキストパス。
+	 */
+	public String getContextPath() {
+		return context_path;
+	}
+
+	/**
+	 * コンテキストパスの設定。
+	 * @param str コンテキストパス。
+	 */
+	protected void setContextPath( String str ) {
+		context_path = str;
+	}
+
 	/**
 	 * サーブレット設定。
 	 * @param sv サーブレット。
@@ -297,7 +315,8 @@ public class RequestParameter implements Serializable {
 			setURI( new URI( buf.toString() ) );
 
 			String[]	path = getURI().getPath().split( "/" );
-			String[]	cont = request.getContextPath().split( "/" );
+			setContextPath( request.getContextPath() );
+			String[]	cont = getContextPath().split( "/" );
 			String	last = cont[cont.length - 1];
 			int	start = 0;
 			for ( ; start < path.length; start++ ) {
@@ -446,6 +465,19 @@ public class RequestParameter implements Serializable {
 		return in_cookie;
 	}
 
+	/**
+	 * 指定クッキーの取得。
+	 * @param name クッキーの名称。
+	 * @return 名称が一致するクッキー。無ければ null。
+	 */
+	public Cookie getCookie( String name ) {
+		if ( name == null )	return null;
+		for ( int  i = 0; i < in_cookie.length; i++ ) {
+			if ( in_cookie[i].getName().equals( name ) )	return in_cookie[i];
+		}
+		return null;
+	}
+
 	private void setRequestItem( String k, String v, File fb, String mime, boolean f ) {
 		RequestItem	item = item_map.get( k );
 		if ( item == null ) {
@@ -531,20 +563,27 @@ public class RequestParameter implements Serializable {
 	 */
 	public int getHistoryKey( String name ) {
 		HttpSession	session = getSession();
-		if ( session == null )	return -1;
-		SessionData	data = (SessionData)session.getAttribute( Supervisor.SESSION_DATA );
+		SessionData	data = null;
+		if ( session != null ) {
+			data = (SessionData)session.getAttribute( Supervisor.SESSION_DATA );
+		}
+		if ( data == null )	data = Supervisor.getSessionData( getSupervisor() );
+		if ( data == null )	return -1;
 		return data.hist.getKey( name );
 	}
 
 	/**
 	 * 履歴の取得。
 	 * @param key 画面遷移履歴キー。
-	 * @return 履歴。セッションが発生していない場合はnull。
+	 * @return 履歴。
 	 */
 	public History getHistory( int key ) {
 		HttpSession	session = getSession();
-		if ( session == null )	return null;
-		SessionData	data = (SessionData)session.getAttribute( Supervisor.SESSION_DATA );
+		SessionData	data = null;
+		if ( session != null ) {
+			data = (SessionData)session.getAttribute( Supervisor.SESSION_DATA );
+		}
+		if ( data == null )	data = Supervisor.getSessionData( getSupervisor() );
 		if ( data == null )	return null;
 		return data.hist.get( key );
 	}
@@ -563,6 +602,7 @@ public class RequestParameter implements Serializable {
 	 * @return リクエスト項目。
 	 */
 	public RequestItem getItem( String key ) {
+		if ( key == null )	return null;
 		return item_map.get( key );
 	}
 
