@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -10,6 +10,8 @@ package paraselene.supervisor;
 import java.util.*;
 import java.io.*;
 import paraselene.*;
+import paraselene.tag.*;
+import paraselene.ui.*;
 import paraselene.ajax.*;
 import paraselene.ajax.data.*;
 
@@ -137,6 +139,49 @@ class Child extends Thread implements DaemonChannel {
 	public void commitPage( RequestParameter req, AjaxForward ... fw ) {
 	}
 
+	private static PageHooker[] sort( PageHooker[] src ) {
+		Tag[]	tag = new Tag[src.length];
+		for ( int i = 0; i < src.length; i++ ) {
+			tag[i] = (Tag)src[i];
+		}
+		tag = Tag.sort( tag );
+		for ( int i = 0; i < src.length; i++ ) {
+			src[i] = (PageHooker)tag[i];
+		}
+		return src;
+	}
+
+	private static Forward input( Page page, RequestParameter req, Forward fw ) throws Exception {
+		if ( page == null )	return fw;
+		PageHooker[]	hook = sort( page.getPageHooker() );
+		for ( int i = 0; i < hook.length; i++ ) {
+			fw = hook[i].beforeInput( page, req, fw );
+		}
+		return page.input( req, fw );
+	}
+
+	private static Page output( Page page, Page from, RequestParameter req ) throws Exception {
+		if ( page == null )	return null;
+		Page	old = page;
+		page.beginning( req );
+		page = page.output( from, req );
+		if ( page == null )	page = old;
+
+		PageBypassedDiv[]	div = page.getPageBypassedDiv();
+		for ( int i = 0; i < div.length; i++ ) {
+			div[i].setView( output( div[i].getBypassPage(), from, req ) );
+		}
+
+		PageHooker[]	hook = sort( page.getPageHooker() );
+		for ( int i = 0; i < hook.length; i++ ) {
+			hook[i].afterOutput( from, old, req );
+		}
+		if ( old != page ) {
+			old.resetOnLoadScript();
+		}
+		return page;
+	}
+
 	private void userFunction() {
 		History	hist = null;
 		if ( main_page == null )	return;
@@ -149,12 +194,13 @@ class Child extends Thread implements DaemonChannel {
 			}
 			catch( Exception e ) {
 			}
+
 			if ( call_input ) {
-				forward = main_page.input( request, forward );
+				forward = input( main_page, request, forward );
 			}
 			else {
-				main_page.beginning( request );
-				param_page = main_page.output( param_page, request );
+				param_page = output( main_page, param_page, request );
+
 			}
 		}
 		catch( Throwable e ) {
