@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -47,7 +47,7 @@ class SessionData implements Serializable, HttpSessionBindingListener {
 		private static final long serialVersionUID = 25L;
 		volatile boolean flag = true;
 	}
-	private static final String	DUMMY = "paraselene_dmy";
+	private static final String	DUMMY = paraselene.tag.form.Form.FORM_DUMMY;
 	private ArrayList<String>	cache_dummy_key = new ArrayList<String>();
 	private ArrayList<CacheAjax>	cache_ajax = new ArrayList<CacheAjax>();
 	private HashMap<String, Waiter>	working_ajax = new HashMap<String, Waiter>();
@@ -59,6 +59,11 @@ class SessionData implements Serializable, HttpSessionBindingListener {
 		if ( key == null )	return null;
 		key = key.trim();
 		if ( key.isEmpty() )	return null;
+		try{
+			int	dmy = Integer.parseInt( key );
+			if ( dmy < 0 )	return null;
+		}
+		catch( Exception e ){}
 		return key;
 	}
 
