@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -228,25 +228,56 @@ public abstract class Supervisor extends HttpServlet {
 		return uri;
 	}
 
-	static boolean isMyHost( String uri ) {
-		DataHolder	holder = null;
-		synchronized( data_map_th ) {
-			holder = data_map_th.get( Thread.currentThread() );
-		}
-		if ( holder == null )	return false;
+	private static boolean isMyHost( DataHolder holder, String uri ) {
+		if ( holder == null || uri == null )	return false;
 		try {
 			URI	test = new URI( URIValue.encode( uri, "utf-8" ) );
 			if ( !test.isAbsolute() )	return true;
+			String	sch = test.getScheme();
+			if ( sch != null ) {
+				if ( !"https".equals( sch ) && !"http".equals( sch ) )	return false;
+			}
 			String	host = test.getHost();
 			if ( host == null )	return true;
 			if ( host.equals( holder.req.getServerName() ) )	return true;
 		}
 		catch( Exception e ){
-			Option.debug( e );
 		}
 		return false;
 	}
 
+	static boolean isMyHost( String uri ) {
+		DataHolder	holder = null;
+		synchronized( data_map_th ) {
+			holder = data_map_th.get( Thread.currentThread() );
+		}
+		return isMyHost( holder, uri );
+	}
+
+	/**
+	 * URIが自ホストであるか？
+	 * @param uri 検査する URI。
+	 * @return true:自ホスト、false:他のホスト。
+	 */
+	public boolean isMyHostURI( String uri ) {
+		DataHolder	holder = null;
+		synchronized( data_map_th ) {
+			holder = data_map_th.get( Thread.currentThread() );
+			if ( holder == null )	holder = data_map_sv.get( this );
+		}
+		return isMyHost( holder, uri );
+	}
+
+	static SessionData getSessionData( Supervisor sv ) {
+		DataHolder	holder = null;
+		synchronized( data_map_th ) {
+			holder = data_map_th.get( Thread.currentThread() );
+			if ( holder == null )	holder = data_map_sv.get( sv );
+		}
+		if ( holder == null )	return null;
+		return holder.data;
+	}
+
 	/**
 	 * ページファクトリーの取得。
 	 * @return ページファクトリー。
@@ -373,10 +404,11 @@ public abstract class Supervisor extends HttpServlet {
 			}
 			last_page = path_data.from;
 			next_page = path_data.to;
+			RequestItem	ukey = req.getItem( Form.PAGE_UKEY );
 			RequestItem	ajax = req.getItem( A_AJAX_KEY );
 			if ( ajax == null )	ajax = req.getItem( Form.FORM_AJAX );
-			if ( ajax != null ) {
-				String	call_key = ajax.getValue( 0 );
+			if ( ajax != null || ukey != null ) {
+				String	call_key = ( ajax != null )?	ajax.getValue( 0 ):	ukey.getValue( 0 );
 				Option.trace( "call_key=%s", call_key );
 				Page	sub = data.hist.get( (path_data.from_histry != null)? path_data.from_histry: HistorySet.ROOT ).getBrowsingPage( call_key );
 				if ( sub != null ) {
@@ -467,6 +499,11 @@ public abstract class Supervisor extends HttpServlet {
 				if ( path_data.to_histry != null ){
 					to_key = path_data.to_histry;
 				}
+				Page	do_page = last_page.reflect( req );
+				if ( do_page != null && do_page != last_page ) {
+					last_page = do_page;
+				}
+				last_page.setRequestParameter( req );
 				if ( next_page != null ) {
 					if ( from_key != to_key ) {
 						fw = new Forward( next_page.getID(), true, false );
@@ -489,8 +526,6 @@ public abstract class Supervisor extends HttpServlet {
 					}
 					fact.returnPage( next_page );
 				}
-				last_page.setRequestParameter( req );
-				last_page.reflect( req );
 				Option.trace( "%s input(%s) call", last_page.getID(), fw );
 				fw = SandBox.input( last_page, req, fw );
 				session = req.getSession();
@@ -537,7 +572,8 @@ public abstract class Supervisor extends HttpServlet {
 					Ajax.add( session, key, oh.getPostBack() );
 					Page	ajax_page = getPageFactory().getPage( PAGE_ID_FORM_JSON );
 					ajax_page.output( last_page, req );
-					write( new Page[]{ ajax_page, ajax_page }, session_f, fw, holder );
+					data.redirect( new Page[]{ ajax_page, ajax_page }, fw, req ).Action( this, res );
+					//write( new Page[]{ ajax_page, ajax_page }, session_f, fw, holder );
 					Option.trace( "FORM postback %s", key );
 					return;
 				}
@@ -712,7 +748,7 @@ public abstract class Supervisor extends HttpServlet {
 				w.writeTo( res );
 			}
 		}
-		data.hist.get( next_page.getHistoryKey() ).add( next_page );
+		//data.hist.get( next_page.getHistoryKey() ).add( next_page );
 	}
 
 	private boolean gzip( RequestParameter param, HttpServletRequest req, HttpServletResponse res, Page page, SessionData data, String mime )  throws Exception {
