@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -10,6 +10,7 @@ import java.util.*;
 import java.io.*;
 import java.net.*;
 import java.math.*;
+import javax.servlet.http.*;
 import paraselene.*;
 import paraselene.tag.form.*;
 import paraselene.supervisor.*;
@@ -19,7 +20,7 @@ import paraselene.ajax.data.*;
  * HTML汎用タグ。
  */
 public class Tag extends HTMLPart {
-	private static final Page	null_page = new NullPage(){
+	static final Page	null_page = new NullPage(){
 		public Forward input( RequestParameter req, Forward fw ) throws PageException {return null;}
 	};
 
@@ -101,7 +102,7 @@ public class Tag extends HTMLPart {
 		set_modify();
 	}
 
-	private void getModifiedTag( ArrayList<Tag> list ) {
+	void getModifiedTag( ArrayList<Tag> list ) {
 		if ( dirty_f ) {
 			list.add( this );
 			return;
@@ -545,6 +546,7 @@ public class Tag extends HTMLPart {
 	 * @param d 文書。
 	 */
 	public void addHTMLPart( int idx, HTMLPart d ) {
+		if ( this instanceof VirtualTag || d instanceof VirtualTag )	return;
 		setModify();
 		doc.add( idx, d );
 		if ( d instanceof Tag ) {
@@ -1204,57 +1206,18 @@ public class Tag extends HTMLPart {
 		setEventHandler( triger, ano );
 	}
 
-/************
-	protected void setButtonAjaxHandler( String triger, String function, Object ... param ) {
-		Page	page = getAssignedPage();
-		if ( !page.isAjax() )	return;
-		Attribute	attr = getAttribute( triger );
-		if ( attr != null )	return;
-		setAttribute( new Attribute( triger, String.format( function, (Object[])param ) ) );
+	/**
+	 * class属性の取得。空白文字で分割したものを返します。
+	 * @return 無ければ 0 個の配列。
+	 */
+	public String[]	getClassAttribute() {
+		Attribute	cls = getAttribute( "class" );
+		if ( cls == null )	return new String[0];
+		String	str = cls.getString();
+		if ( str == null )	return new String[0];
+		return str.split( "\\p{Space}" );
 	}
 
-	protected void setHTTPAjaxHandler( String url_attr, String triger, String function ) {
-		if ( !isAjaxEnable() || getAttribute( "target" ) != null )	return;
-		Attribute	attr = getAttribute( url_attr );
-		if ( attr == null )	return;
-		URIValue	v = (URIValue)attr.get();
-		if ( v == null )	return;
-		if ( v.isParaselenePagePath() ) {
-			setButtonAjaxHandler( triger, function, getAssignedPage().getUniqueKey() );
-		}
-	}
-
-	protected void setHttpAjaxEventListener( PrintWriter w, HTMLPart.StringMode mode, String func ) {
-		setButtonAjaxEventListener( w, mode, func, null );
-	}
-
-	protected void setButtonAjaxEventListener( PrintWriter w, HTMLPart.StringMode mode, String func, String param ) {
-		Page	page = getAssignedPage();
-		if ( !page.isAjax() )	return;
-		StringBuilder	buf;
-		if ( param == null ) {
-			if ( !isAjaxEnable() || getAttribute( "target" ) != null )	return;
-			buf = new StringBuilder( "'" );
-			buf = buf.append( page.getUniqueKey() );
-			buf = buf.append( "'" );
-			param = buf.toString();
-		}
-		buf = new StringBuilder( "'" );
-		buf = buf.append( getIDAttribute() );
-		buf = buf.append( "'" );
-		String	id = buf.toString();
-		String	s = String.format( func, id, param );
-		if ( w instanceof JsonWriter ) {
-			JsonWriter	jw = (JsonWriter)w;
-			jw.addScript( s );
-			return;
-		}
-		Tag	tag = makeScript( s );
-		tag.write( w, mode );
-	}
-*******/
-
-
 	/**
 	 * JavaScript を安全に記述するための script タグを生成します。<br>
 	 * このタグの内容を変更しても、Feedback として通知されないので注意して下さい。
@@ -1269,5 +1232,99 @@ public class Tag extends HTMLPart {
 		tag.addHTMLPart( new paraselene.ajax.data.JavaScript( script ) );
 		return tag;
 	}
+
+	private String makeCookieName() throws IOException {
+		String	name = getNameAttribute();
+		if ( name == null )	throw new IOException( "no name attribute." );
+		Page	page = getAssignedPage();
+		if ( page == null )	throw new IOException( "no page assigned." );
+		String	pid = page.getID().toString();
+		StringBuilder	buf = new StringBuilder( "paraselene" );
+		buf = buf.append( "-" );
+		buf = buf.append( pid );
+		buf = buf.append( "-" );
+		buf = buf.append( name );
+		return buf.toString();
+	}
+
+	private Object getCookieValue( String str ) throws IOException, ClassNotFoundException {
+		ByteArrayInputStream	bais = new ByteArrayInputStream(  new BigInteger( str, Character.MAX_RADIX ).toByteArray() );
+		ObjectInputStream	ois = new ObjectInputStream( bais );
+		return ois.readObject();
+	}
+
+	private String makeCookieValue( Object obj ) throws IOException {
+		ByteArrayOutputStream	baos = new ByteArrayOutputStream();
+		ObjectOutputStream	oos = new ObjectOutputStream( baos );
+		oos.writeObject( obj );
+		return new BigInteger( baos.toByteArray() ).toString( Character.MAX_RADIX );
+	}
+
+	/**
+	 * このタグのクッキー設定値を返します。
+	 * @param req リクエスト内容。
+	 * @return クッキー設定値。クッキーが存在しない場合、null。
+	 */
+	protected Object getCookieValue( RequestParameter req ) throws IOException, ClassNotFoundException {
+		Cookie	c = req.getCookie( makeCookieName() );
+		if ( c == null )	return null;
+		return getCookieValue( c.getValue() );
+	}
+
+	/**
+	 * このタグの設定値を保存するためのクッキーを生成。
+	 * @param req リクエスト内容。
+	 * @param val 設定値。
+	 * @param expiry クッキー存続時間(秒)。
+	 * @param ssl_f true:HTTPSの場合のみクッキーを返すようにする、false:全てのプロトコルでクッキーを返すようにする。
+	 * @return クッキー。
+	 */
+	protected Cookie makeCookieValue( RequestParameter req, Object val, int expiry, boolean ssl_f ) throws IOException {
+		Cookie	c = new Cookie( makeCookieName(), makeCookieValue( val ) );
+		c.setMaxAge( expiry );
+		c.setPath( req.getContextPath() );
+		c.setSecure( ssl_f );
+		return c;
+	}
+
+	/**
+	 * クッキーからの値を復元。<br>
+	 * 自身のタグに対するクッキーから、以前の値を setValueString() します。<br>
+	 * ただし、isEmptyValue() が false なら処理しません。
+	 * @param req リクエスト内容。
+	 */
+	public void importFromCookie( RequestParameter req ) {
+		if ( !isEmptyValue() )	return;
+		try {
+			String	val = (String)getCookieValue( req );
+			if ( val != null )	setValueString( val );
+		}
+		catch( Exception e ){}
+	}
+
+	/**
+	 * クッキーへ値の保存。<br>
+	 * クッキーへ getValueString() したものを設定します。<br>
+	 * ただし、isEmptyValue() が true なら、クッキーへ保存しません。<br>
+	 * また、<ul>
+	 * <li>ページに属していないタグ
+	 * <li>name 属性を持たないタグ
+	 * </ul>もクッキーへ保存できません。
+	 * @param req リクエスト内容。
+	 * @param fw 設定先インスタンス。
+	 * @param expiry クッキー存続時間(秒)。
+	 * @param ssl_f true:HTTPSの場合のみクッキーを返すようにする、
+	 * false:全てのプロトコルでクッキーを返すようにする。
+	 * @return fw をそのまま返します。
+	 */
+	public Forward exportToCookie( RequestParameter req, Forward fw, int expiry, boolean ssl_f ) {
+		if ( isEmptyValue() )	return fw;
+		try {
+			fw.addCookie( makeCookieValue( req, getValueString(), expiry, ssl_f ) );
+		}
+		catch( Exception e ){}
+		return fw;
+	}
+
 }
 
