@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -458,5 +458,18 @@ public abstract class Control extends Tag {
 			throw new ControlException( mes, this );
 		}
 	}
+
+	/**
+	 * 所属する Form を返します。
+	 * @return 所属Form。所属していなければ null。
+	 */
+	public Form getForm() {
+		Tag	tag = this;
+		while( tag != null ) {
+			if ( tag instanceof Form )	return (Form)tag;
+			tag = tag.getOuterTag();
+		}
+		return null;
+	}
 }
 
