@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -14,6 +14,7 @@ import javax.servlet.http.*;
 import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
+import paraselene.ui.*;
 
 class FormSeq {
 	private volatile int seq = 0;
@@ -29,7 +30,7 @@ class FormSeq {
 /**
  * FORMタグ。
  */
-public class Form extends Tag {
+public class Form extends Tag implements PageHooker {
 	private static final long serialVersionUID = 2L;
 	/**
 	 * 予約済みHIDDEN項目のname。
@@ -43,6 +44,10 @@ public class Form extends Tag {
 	 * 予約済みHIDDEN項目のname。
 	 */
 	public static final String FORM_DUMMY = "paraselene_dummy";
+	/**
+	 * 予約済みHIDDEN項目のname。
+	 */
+	public static final String PAGE_UKEY = "paraselene_page_ukey";
 	/**
 	 * マルチパート。
 	 */
@@ -59,6 +64,7 @@ public class Form extends Tag {
 		skip_name.put( FORM_ID, FORM_ID );
 		skip_name.put( FORM_AJAX, FORM_AJAX );
 		skip_name.put( FORM_DUMMY, FORM_DUMMY );
+		skip_name.put( PAGE_UKEY, PAGE_UKEY );
 	}
 
 	/**
@@ -75,7 +81,7 @@ public class Form extends Tag {
 		return map.get( name ) == cont;
 	}
 
-	private Input	dummy = Input.createHidden( FORM_DUMMY, "-1" );
+	private Input	page_ukey = Input.createHidden( PAGE_UKEY, "-1" );
 
 	/**
 	 * コンストラクタ。
@@ -86,7 +92,8 @@ public class Form extends Tag {
 		Input[]	id = new Input[] {
 			Input.createHidden( FORM_ID, seq ),
 			Input.createHidden( FORM_AJAX, "-1" ),
-			dummy
+			Input.createHidden( FORM_DUMMY, "-1" ),
+			page_ukey
 		};
 		addHTMLPart( id );
 		fixed_f = true;
@@ -103,7 +110,11 @@ public class Form extends Tag {
 		super.addHTMLPart( idx, d );
 	}
 
-	public void write( PrintWriter w, StringMode mode ) {
+	public Forward beforeInput( Page page, RequestParameter req, Forward fw ) throws Exception {
+		return fw;
+	}
+
+	public void afterOutput( Page from, Page to, RequestParameter req ) throws Exception {
 		StringBuilder	buf = new StringBuilder( "function(ev){" );
 		buf = buf.append( "if(paraselene_form_kill)return;" );
 		buf = buf.append( "if(!paraselene_agency_form(paraselene_get_id('" );
@@ -112,11 +123,7 @@ public class Form extends Tag {
 		buf = buf.append( getAssignedPage().getUniqueKey() );
 		buf = buf.append( "'))paraselene_kill_event(ev);}" );
 		setAnonymousEventHandler( "submit", buf.toString() );
-		//setHTTPAjaxHandler( "action", "onsubmit", "return paraselene_agency_form(this,'%s');" );
-		dummy.setValueString( form_seq.getSeq() );
-		//makeID();
-		super.write( w, mode );
-		//setHttpAjaxEventListener( w, mode, "paraselene_form_handler(%s,%s)" );
+		page_ukey.setValueString( getAssignedPage().getUniqueKey() );
 	}
 
 	protected Tag newReplica() {
@@ -261,7 +268,7 @@ public class Form extends Tag {
 			Button	button = null;
 			if ( cont[i] instanceof Button ) {
 				button = (Button)cont[i];
-				button.setPushed( false );
+				button.setClicked( false );
 			}
 			RequestItem	item = req.getItem( name );
 			if ( item == null ) {
@@ -285,7 +292,7 @@ public class Form extends Tag {
 				continue;
 			}
 			if ( button != null ) {
-				button.setPushed( true );
+				button.setClicked( true );
 				set.add( button );
 			}
 			else if ( cbox != null ) {
