@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009  Akira Terasaki
+ * Copyright (c) 2009, 2010  Akira Terasaki
  * このファイルは同梱されている License.txt に定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -9,6 +9,8 @@ package paraselene;
 import paraselene.supervisor.*;
 import java.io.*;
 import java.net.*;
+import java.util.*;
+import java.util.zip.*;
 import java.util.jar.*;
 
 /**
@@ -107,19 +109,40 @@ public class Version {
 		return JAR_PATH;
 	}
 
-	/**
-	 * リソースの取得。
-	 * @param name リソースの名前。
-	 * @return リソース。取得に失敗すると null を返します。
-	 */
-	public static InputStream getResource( String name ) {
+	private static String fixName( String name ) {
 		String[]	test = name.split( "\\\\" );
 		StringBuilder	buf = new StringBuilder( test[0] );
 		for ( int i = 1; i < test.length; i++ ) {
 			buf = buf.append( "/" );
 			buf = buf.append( test[i] );
 		}
-		name = buf.toString();
+		return buf.toString();
+	}
+
+	/**
+	 * リソースの更新時間の取得。
+	 * @param name リソースの名前。
+	 * @return 更新時間。
+	 */
+	public static HTTPDate getLastModified( String name ) {
+		name = fixName( name );
+		try {
+			JarFile	file = new JarFile( getJarPath() );
+			return new HTTPDate( new Date( file.getEntry( name ).getTime() ) );
+		}
+		catch( Exception e ) {
+			Option.debug( e );
+		}
+		return new HTTPDate( new Date( getJarPath().lastModified() ) );
+	}
+
+	/**
+	 * リソースの取得。
+	 * @param name リソースの名前。
+	 * @return リソース。取得に失敗すると null を返します。
+	 */
+	public static InputStream getResource( String name ) {
+		name = fixName( name );
 
 		ClassLoader	ld = Version.class.getClassLoader();
 		if ( ld != null ) {
